from grpclib.client import Channel
from qua.grpc.io.qualang.api.v1 import InfoServiceStub
from qua.info import QuaMachineInfo, ImplementationInfo


class QuaClient:
    def __init__(self, *, host: str = "127.0.0.1", port: int = 80, **kwargs) -> None:
        self._host = host
        self._port = port
        self._channel_factory = kwargs.get("grpc_channel_factory", None)
        super().__init__()

    def _create_channel(self):
        if self._channel_factory is not None:
            return self._channel_factory()
        else:
            return Channel(host=self._host, port=self._port)

    async def get_server_info(self):
        with self._create_channel() as channel:
            service = InfoServiceStub(channel)
            response = await service.get_info()
            return QuaMachineInfo(
                capabilities=response.capabilities,
                implementation=ImplementationInfo(
                    name=response.implementation.name,
                    version=response.implementation.version,
                    url=response.implementation.url,
                ),
            )



import dataclasses
from typing import List


@dataclasses.dataclass
class ImplementationInfo:
    name: str
    version: str
    url: str


@dataclasses.dataclass
class QuaMachineInfo:
    capabilities: List[str]
    implementation: ImplementationInfo



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['qua',
 'qua.grpc',
 'qua.grpc.io',
 'qua.grpc.io.qualang',
 'qua.grpc.io.qualang.api']

package_data = \
{'': ['*']}

install_requires = \
['betterproto>=1.2.5,<2.0.0', 'grpclib>=0.4.2,<0.5.0']

setup_kwargs = {
    'name': 'qua',
    'version': '0.1.0rc1',
    'description': 'SDK to interact with a quantum computer at the pulse level',
    'long_description': None,
    'author': 'Tal Shani',
    'author_email': 'tal@quantum-machines.co',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: io/qualang/api/v1/info_service.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import betterproto
import grpclib


@dataclass
class GetInfoRequest(betterproto.Message):
    pass


@dataclass
class GetInfoResponse(betterproto.Message):
    implementation: "ImplementationDetails" = betterproto.message_field(1)
    capabilities: List[str] = betterproto.string_field(2)


@dataclass
class ImplementationDetails(betterproto.Message):
    name: str = betterproto.string_field(1)
    version: str = betterproto.string_field(2)
    url: str = betterproto.string_field(3)


class InfoServiceStub(betterproto.ServiceStub):
    async def get_info(self) -> GetInfoResponse:
        request = GetInfoRequest()

        return await self._unary_unary(
            "/io.qualang.api.v1.InfoService/GetInfo",
            request,
            GetInfoResponse,
        )
